package com.qmth.themis.exam.api;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.qmth.themis.business.annotation.ApiJsonObject;
import com.qmth.themis.business.annotation.ApiJsonProperty;
import com.qmth.themis.business.cache.RedisKeyHelper;
import com.qmth.themis.business.constant.SystemConstant;
import com.qmth.themis.business.dto.MqDto;
import com.qmth.themis.business.entity.TIeExamInvigilateCallLog;
import com.qmth.themis.business.enums.MonitorStatusSourceEnum;
import com.qmth.themis.business.enums.MonitorVideoSourceEnum;
import com.qmth.themis.business.enums.MqTagEnum;
import com.qmth.themis.business.enums.MqTopicEnum;
import com.qmth.themis.business.service.MqDtoService;
import com.qmth.themis.business.service.TIeExamInvigilateCallLogService;
import com.qmth.themis.business.util.RedisUtil;
import com.qmth.themis.common.enums.ExceptionResultEnum;
import com.qmth.themis.common.exception.BusinessException;
import com.qmth.themis.common.util.Result;
import com.qmth.themis.common.util.ResultUtil;
import io.swagger.annotations.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Objects;

/**
 * @Description: oe监考监控通话信息 前端控制器
 * @Param:
 * @return:
 * @Author: wangliang
 * @Date: 2020/6/25
 */
@Api(tags = "oe监考监控通话信息Controller")
@RestController
@RequestMapping("/${prefix.url.exam}/monitor")
public class TIeInvigilateCallLogOeController {
    private final static Logger log = LoggerFactory.getLogger(TIeInvigilateCallLogOeController.class);

    @Resource
    TIeExamInvigilateCallLogService tIeExamInvigilateCallLogService;

    @Resource
    RedisUtil redisUtil;

    @Resource
    MqDtoService mqDtoService;

    @ApiOperation(value = "监控观看地址更新接口")
    @RequestMapping(value = "/live_url", method = RequestMethod.POST)
    @ApiResponses({@ApiResponse(code = 200, message = "{\"success\":true}", response = Result.class)})
    @Transactional
    public Result liveUrl(@ApiJsonObject(name = "liveUrlOe", value = {
            @ApiJsonProperty(key = "recordId", type = "long", example = "1", description = "考试记录id", required = true),
            @ApiJsonProperty(key = "source", description = "监考视频源", required = true),
            @ApiJsonProperty(key = "liveUrl", description = "观看地址", required = true)
    }) @ApiParam(value = "监控信息", required = true) @RequestBody Map<String, Object> mapParameter) {
        if (Objects.isNull(mapParameter.get("recordId")) || Objects.equals(mapParameter.get("recordId"), "")) {
            throw new BusinessException(ExceptionResultEnum.RECORD_ID_IS_NULL);
        }
        Long recordId = null;
        recordId = Long.parseLong(String.valueOf(mapParameter.get("recordId")));
        if (Objects.isNull(mapParameter.get("source")) || Objects.equals(mapParameter.get("source"), "")) {
            throw new BusinessException("监考视频源不能为空");
        }
        MonitorVideoSourceEnum source = MonitorVideoSourceEnum.valueOf(String.valueOf(mapParameter.get("source")).toUpperCase());
        if (Objects.isNull(mapParameter.get("liveUrl")) || Objects.equals(mapParameter.get("liveUrl"), "")) {
            throw new BusinessException("观看地址不能为空");
        }
        String liveUrl = String.valueOf(mapParameter.get("liveUrl"));
        TIeExamInvigilateCallLog tIeExamInvigilateCallLog = new TIeExamInvigilateCallLog(recordId, source, liveUrl, MonitorStatusSourceEnum.INIT);
        //获取考试记录缓存
        Map<String, Object> objectMap = redisUtil.getHashEntries(RedisKeyHelper.examRecordCacheKey(recordId));
        objectMap.put(SystemConstant.MONITOR_LIVE_URL_ + source.name(), liveUrl);
        objectMap.put(SystemConstant.MONITOR_STATUS_ + source.name(), tIeExamInvigilateCallLog.getStatus().name());
        redisUtil.setForHash(RedisKeyHelper.examRecordCacheKey(recordId), objectMap);
        //监考监控通话信息 发送mq start
        MqDto mqDto = new MqDto(MqTopicEnum.themisTopic.getCode(), MqTagEnum.monitorLog.name(), tIeExamInvigilateCallLog, MqTagEnum.monitorLog, String.valueOf(tIeExamInvigilateCallLog.getId()), source.name());
        mqDtoService.assembleSendOneWayMsg(mqDto);
        //监考监控通话信息 发送mq end
        return ResultUtil.ok(SystemConstant.SUCCESS);
    }

    @ApiOperation(value = "发送通话申请接口")
    @RequestMapping(value = "/call/apply", method = RequestMethod.POST)
    @ApiResponses({@ApiResponse(code = 200, message = "{\"success\":true}", response = Result.class)})
    @Transactional
    public Result callApply(@ApiJsonObject(name = "callApplyOe", value = {
            @ApiJsonProperty(key = "recordId", type = "long", example = "1", description = "考试记录id", required = true),
    }) @ApiParam(value = "监控信息", required = true) @RequestBody Map<String, Object> mapParameter) {
        if (Objects.isNull(mapParameter.get("recordId")) || Objects.equals(mapParameter.get("recordId"), "")) {
            throw new BusinessException(ExceptionResultEnum.RECORD_ID_IS_NULL);
        }
        Long recordId = Long.parseLong(String.valueOf(mapParameter.get("recordId")));
        //获取考试记录缓存
        Map<String, Object> objectMap = redisUtil.getHashEntries(RedisKeyHelper.examRecordCacheKey(recordId));
        String liveUrl = null;
        MonitorVideoSourceEnum source = null;
        if (Objects.nonNull(objectMap.get(SystemConstant.MONITOR_LIVE_URL_ + MonitorVideoSourceEnum.MOBILE_FIRST.name()))) {
            source = MonitorVideoSourceEnum.MOBILE_FIRST;
            liveUrl = String.valueOf(objectMap.get(SystemConstant.MONITOR_LIVE_URL_ + MonitorVideoSourceEnum.MOBILE_FIRST.name()));
        } else if (Objects.nonNull(objectMap.get(SystemConstant.MONITOR_LIVE_URL_ + MonitorVideoSourceEnum.MOBILE_SECOND.name()))) {
            source = MonitorVideoSourceEnum.MOBILE_SECOND;
            liveUrl = String.valueOf(objectMap.get(SystemConstant.MONITOR_LIVE_URL_ + MonitorVideoSourceEnum.MOBILE_SECOND.name()));
        }
        objectMap.put(SystemConstant.MONITOR_STATUS_ + source, MonitorStatusSourceEnum.START.name());
        TIeExamInvigilateCallLog tIeExamInvigilateCallLog = new TIeExamInvigilateCallLog(recordId, source, liveUrl, MonitorStatusSourceEnum.START);
        redisUtil.setForHash(RedisKeyHelper.examRecordCacheKey(recordId), objectMap);
        //监考监控通话信息 发送mq start
        MqDto mqDto = new MqDto(MqTopicEnum.themisTopic.getCode(), MqTagEnum.monitorLog.name(), tIeExamInvigilateCallLog, MqTagEnum.monitorLog, String.valueOf(tIeExamInvigilateCallLog.getId()), source.name());
        mqDtoService.assembleSendOneWayMsg(mqDto);
        //监考监控通话信息 发送mq end
        return ResultUtil.ok(SystemConstant.SUCCESS);
    }

    @ApiOperation(value = "撤销通话申请接口")
    @RequestMapping(value = "/call/cancel", method = RequestMethod.POST)
    @ApiResponses({@ApiResponse(code = 200, message = "{\"success\":true}", response = Result.class)})
    @Transactional
    public Result callCancel(@ApiJsonObject(name = "callCancelOe", value = {
            @ApiJsonProperty(key = "recordId", type = "long", example = "1", description = "考试记录id", required = true),
    }) @ApiParam(value = "监控信息", required = true) @RequestBody Map<String, Object> mapParameter) {
        if (Objects.isNull(mapParameter.get("recordId")) || Objects.equals(mapParameter.get("recordId"), "")) {
            throw new BusinessException(ExceptionResultEnum.RECORD_ID_IS_NULL);
        }
        Long recordId = Long.parseLong(String.valueOf(mapParameter.get("recordId")));
        //获取考试记录缓存
        Map<String, Object> objectMap = redisUtil.getHashEntries(RedisKeyHelper.examRecordCacheKey(recordId));
        String liveUrl = null;
        MonitorVideoSourceEnum source = null;
        if (Objects.nonNull(objectMap.get(SystemConstant.MONITOR_LIVE_URL_ + MonitorVideoSourceEnum.MOBILE_FIRST.name()))) {
            source = MonitorVideoSourceEnum.MOBILE_FIRST;
            liveUrl = String.valueOf(objectMap.get(SystemConstant.MONITOR_LIVE_URL_ + MonitorVideoSourceEnum.MOBILE_FIRST.name()));
        } else if (Objects.nonNull(objectMap.get(SystemConstant.MONITOR_LIVE_URL_ + MonitorVideoSourceEnum.MOBILE_SECOND.name()))) {
            source = MonitorVideoSourceEnum.MOBILE_SECOND;
            liveUrl = String.valueOf(objectMap.get(SystemConstant.MONITOR_LIVE_URL_ + MonitorVideoSourceEnum.MOBILE_SECOND.name()));
        }
        objectMap.put(SystemConstant.MONITOR_STATUS_ + source, MonitorStatusSourceEnum.STOP.name());
        TIeExamInvigilateCallLog tIeExamInvigilateCallLog = new TIeExamInvigilateCallLog(recordId, source, liveUrl, MonitorStatusSourceEnum.STOP);
        redisUtil.setForHash(RedisKeyHelper.examRecordCacheKey(recordId), objectMap);
        //监考监控通话信息 发送mq start
        MqDto mqDto = new MqDto(MqTopicEnum.themisTopic.getCode(), MqTagEnum.monitorLog.name(), tIeExamInvigilateCallLog, MqTagEnum.monitorLog, String.valueOf(tIeExamInvigilateCallLog.getId()), source.name());
        mqDtoService.assembleSendOneWayMsg(mqDto);
        //监考监控通话信息 发送mq end
        return ResultUtil.ok(SystemConstant.SUCCESS);
    }

    @ApiOperation(value = "监考监控通话查询接口")
    @RequestMapping(value = "/call/list", method = RequestMethod.POST)
    @ApiResponses({@ApiResponse(code = 200, message = "监考监控信息", response = TIeExamInvigilateCallLog.class)})
    public Result callList(@ApiParam(value = "场次id", required = true) @RequestParam Long examActivityId) {
        QueryWrapper<TIeExamInvigilateCallLog> tIeExamInvigilateCallQueryWrapper = new QueryWrapper<>();
        tIeExamInvigilateCallQueryWrapper.lambda().eq(TIeExamInvigilateCallLog::getSource, "apply");
        List<TIeExamInvigilateCallLog> tIeExamInvigilateCallList = tIeExamInvigilateCallLogService.list(tIeExamInvigilateCallQueryWrapper);
        Map map = new HashMap<>();
        map.put(SystemConstant.RECORDS, tIeExamInvigilateCallList);
        return ResultUtil.ok(map);
    }

    @ApiOperation(value = "监考监控通话查询来源接口")
    @RequestMapping(value = "/call/query", method = RequestMethod.POST)
    @ApiResponses({@ApiResponse(code = 200, message = "监考监控信息", response = TIeExamInvigilateCallLog.class)})
    public Result callQuery(@ApiParam(value = "考试记录id", required = true) @RequestParam(required = true) Long recordId) {
        QueryWrapper<TIeExamInvigilateCallLog> tIeExamInvigilateCallQueryWrapper = new QueryWrapper<>();
        tIeExamInvigilateCallQueryWrapper.lambda().eq(TIeExamInvigilateCallLog::getExamRecordId, recordId);
        List<TIeExamInvigilateCallLog> tIeExamInvigilateCallList = tIeExamInvigilateCallLogService.list(tIeExamInvigilateCallQueryWrapper);
        Map map = new HashMap<>();
        map.put(SystemConstant.RECORDS, tIeExamInvigilateCallList);
        return ResultUtil.ok(map);
    }
}
